/////////////////////////////////////////////////////////////////////////////////
//
// Thor C++ Library
// Copyright (c) 2011 Jan Haller
// 
// This software is provided 'as-is', without any express or implied
// warranty. In no event will the authors be held liable for any damages
// arising from the use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not
//    claim that you wrote the original software. If you use this software
//    in a product, an acknowledgment in the product documentation would be
//    appreciated but is not required.
// 
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////

#include <Thor/Detail/ColorImpl.hpp>

#include <cassert>


namespace thor
{
namespace detail
{
	
	ColorGradientTransition::ColorGradientTransition(ColorGradientConvertible& parent)
	: mParent(&parent)
	{
	}

	ColorGradientConvertible& ColorGradientTransition::operator() (const sf::Color& color)
	{
		mParent->AddColor(color);
		return *mParent;
	}

	// ---------------------------------------------------------------------------------------------------------------------------


	ColorGradientConvertible::ColorGradientConvertible(const sf::Color& color)
	: mChild(*this)
	, mColors(1, color)
	, mTransitionTimes()
	{
	}

	ColorGradientTransition& ColorGradientConvertible::operator() (float transitionTime)
	{
		assert(transitionTime >= 0.f);

		mTransitionTimes.push_back(transitionTime);
		return mChild;
	}

	void ColorGradientConvertible::AddColor(const sf::Color& color)
	{
		mColors.push_back(color);
	}

} // namespace detail
} // namespace thor
